/************************************************************************** 
  LICENSE:

    gmLua is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    GMAPI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with GMAPI; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA
***************************************************************************/

#pragma once
#include <iostream>
#include <string>
#include <map>
#include <gmapi.hpp>


extern "C"
{
	#include <lua.h>
	#include <lualib.h>
	#include <lauxlib.h>
}

extern int errorScript;

int _gmCall(lua_State* L);
void _gmError(lua_State* L);
int _getGMFunction(lua_State* L);
int _callGMFunction(lua_State* L);

class luaFile
{
	public:
		luaFile();
		int index;
		lua_State* lua;
		~luaFile();

		void setIndex(std::string name);
		void resetIndex();

		void addScript(std::string name, int callback);
		void call(char *name, int list);

};


int errorScript;

luaFile::luaFile()
{
	index = LUA_GLOBALSINDEX;	
}
luaFile::~luaFile()
{
	resetIndex();
}

void luaFile::setIndex(std::string name)
{
	const char* temp = name.c_str();
	lua_getfield(lua, index, temp);
	int pos = lua_gettop(lua);

	// Was it a table or not?
	if(lua_istable(lua, pos) == 0) {
		// Create the table somehow...
		lua_newtable(lua);				// Push a new table onto the stack
		lua_setfield(lua, index, temp);	// Make the table have a child table
		lua_getfield(lua, index, temp);
		index = lua_gettop(lua);
	}
	else index = pos;
}
void luaFile::resetIndex()
{
	index = LUA_GLOBALSINDEX;
}

void luaFile::addScript(std::string name, int callback)
{
	// Write our callback onto the stack
	lua_pushnumber(lua, callback);
	lua_pushcclosure(lua, _gmCall, 1);
	lua_setfield(lua, index, name.c_str());
}

void luaFile::call(char *name, int list)
{
	lua_getglobal(lua, name);
	int argc = (int)gm::ds_list_size(list);
	if (argc > 0)
	{
		for (int n = 0; n < argc; n++)
		{
			gm::Value val = gm::ds_list_find_value(list, n);

			if (val.type == val.STRING)
			{
				lua_pushstring(lua, val.str.getData());
			} else {
				lua_pushnumber(lua, val.real);
			}
		}
	}

	int s = lua_pcall(lua, argc, 1, 0);

	if (s != 0)
	{
		_gmError(lua);
	} else {
		gm::ds_list_clear(list);

		if (lua_type(lua, -1) == LUA_TSTRING)
		{
			gm::ds_list_add(list, lua_tostring(lua, -1));
		} else {
			gm::ds_list_add(list, lua_tonumber(lua, -1));
		}

		lua_pop(lua, 1);
	}
}

int _gmCall(lua_State* L)
{
	lua_pushvalue(L, lua_upvalueindex(1));
	int argc = lua_gettop(L);

	gm::Value* arg = new gm::Value[argc + 1];
	arg[0] = (int)lua_tonumber(L, argc);
	//gm::show_message(arg[0]);
	//gm::show_message(argc);
	for (int i = 1; i < argc; i++)
	{
		if (lua_type(L, i) == LUA_TSTRING)
		{
			arg[i] = lua_tostring(L, i);
		} else if (lua_type(L, i) == LUA_TNUMBER) {
			arg[i] = lua_tonumber(L, i);
		}
	}

	for(int i = 0;i < argc;++i) {
		//gm::show_message(arg[i]);
	}

	//gm::show_message("executing...");
	gm::Value result = gm::script_execute(argc, arg);//gm::script_execute(callback, arg, argc);
	//gm::show_message("completed execution...");
	delete[] arg;

	if (result.type == gm::Value::STRING)
	{
		lua_pushstring(L, result.str.getData());
	} else {
		lua_pushnumber(L, result.real);
	}
	//gm::show_message("left");

	return 1;
}

void _gmError(lua_State* L)
{
	if (errorScript >= 0)
	{
		gm::Value* arg = new gm::Value[2];
		arg[0] = errorScript;
		arg[1] = lua_tostring(L, -1);
		//MessageBox(0, arg[1].str.getData(), 0, MB_SYSTEMMODAL | MB_ICONERROR);
        gm::script_execute(2, arg);//gm::script_execute(callback, arg, argc);
        delete[] arg;
	}

	lua_pop(L, 1);
}

int _gmPanic(lua_State* L)
{
	_gmError(L);
	return 1;
}

